package vip.xiaonuo.modular.InternalCar.service.impl;/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;


import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.InternalCar.entity.InternalCar;
import vip.xiaonuo.modular.InternalCar.mapper.InternalCarMapper;
import vip.xiaonuo.modular.InternalCar.param.InternalCarAddParam;
import vip.xiaonuo.modular.InternalCar.param.InternalCarEditParam;
import vip.xiaonuo.modular.InternalCar.param.InternalCarIdParam;
import vip.xiaonuo.modular.InternalCar.param.InternalCarPageParam;
import vip.xiaonuo.modular.InternalCar.service.InternalCarService;

import java.util.List;

/**
 * internal_carService接口实现类
 *
 * @author wang
 * @date  2023/12/05 16:29
 **/
@Service
public class InternalCarServiceImpl extends ServiceImpl<InternalCarMapper, InternalCar> implements InternalCarService {
    @Autowired
    private InternalCarMapper carMapper;

    @Override
    public Page<InternalCar> page(InternalCarPageParam internalCarPageParam) {
        QueryWrapper<InternalCar> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(internalCarPageParam.getCarNum())) {
            queryWrapper.lambda().like(InternalCar::getCarNum, internalCarPageParam.getCarNum());
        }
        if(ObjectUtil.isNotEmpty(internalCarPageParam.getUserWay())) {
            queryWrapper.lambda().like(InternalCar::getUserWay, internalCarPageParam.getUserWay());
        }
        if(ObjectUtil.isAllNotEmpty(internalCarPageParam.getSortField(), internalCarPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(internalCarPageParam.getSortOrder());
            queryWrapper.orderBy(true, internalCarPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(internalCarPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(InternalCar::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult add(InternalCarAddParam internalCarAddParam) {
        QueryWrapper<InternalCar> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", internalCarAddParam.getOrgId()).eq("car_num", internalCarAddParam.getCarNum());
        if (ObjectUtil.isEmpty(carMapper.selectOne(queryWrapper))) {
            InternalCar internalCar = BeanUtil.toBean(internalCarAddParam, InternalCar.class);
            this.save(internalCar);
            return CommonResult.ok();
        } else {
            return new CommonResult(500, "车牌号已存在！", false);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult edit(InternalCarEditParam internalCarEditParam) {
        InternalCar internalCar = this.queryEntity(internalCarEditParam.getId());
        QueryWrapper<InternalCar> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", internalCarEditParam.getOrgId())
                .eq(StringUtils.isNotBlank(internalCarEditParam.getCarNum()), "car_num", internalCarEditParam.getCarNum());
        if (ObjectUtil.isEmpty(carMapper.selectOne(queryWrapper))) {
            BeanUtil.copyProperties(internalCarEditParam, internalCar);
            this.updateById(internalCar);
            return CommonResult.ok();
        } else {
            return new CommonResult(500, "修改的车牌号已存在！", false);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<InternalCarIdParam> internalCarIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(internalCarIdParamList, InternalCarIdParam::getId));
    }

    @Override
    public InternalCar detail(InternalCarIdParam internalCarIdParam) {
        return this.queryEntity(internalCarIdParam.getId());
    }

    @Override
    public InternalCar queryEntity(Integer id) {
        InternalCar internalCar = this.getById(id);
        if(ObjectUtil.isEmpty(internalCar)) {
            throw new CommonException("internal_car不存在，id值为：{}", id);
        }
        return internalCar;
    }
}
