/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.dailyInspection.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.ai.dailyInspection.entity.DailyInspection;
import vip.xiaonuo.modular.ai.dailyInspection.entity.vo.DailyInspectionImageVo;
import vip.xiaonuo.modular.ai.dailyInspection.entity.vo.DailyInspectionPageVo;
import vip.xiaonuo.modular.ai.dailyInspection.param.DailyInspectionEditParam;
import vip.xiaonuo.modular.ai.dailyInspection.param.DailyInspectionIdParam;
import vip.xiaonuo.modular.ai.dailyInspection.param.DailyInspectionPageParam;
import vip.xiaonuo.modular.ai.dailyInspection.service.DailyInspectionService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.text.ParseException;

/**
 * 日常巡检控制器
 *
 * @author newspaper
 * @date  2024/01/08 15:58
 */
@Api(tags = "日常巡检控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DailyInspectionController {

    @Resource
    private DailyInspectionService dailyInspectionService;

    /**
     * 获取日常巡检分页
     *
     * @author newspaper
     * @date  2024/01/08 15:58
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取日常巡检分页")
    @GetMapping("/ai/dailyInspection/page")
    public CommonResult<Page<DailyInspectionPageVo>> page(DailyInspectionPageParam dailyInspectionPageParam) {
        return CommonResult.data(dailyInspectionService.page(dailyInspectionPageParam));
    }

    /**
     * 编辑日常巡检
     *
     * @author newspaper
     * @date  2024/01/08 15:58
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("提交日常巡检")
    @CommonLog("提交日常巡检")
    @PostMapping("/ai/dailyInspection/edit")
    public CommonResult<String> edit(@RequestParam @ApiParam(value = "日常巡检id") String id,
                                     @RequestParam @ApiParam(value = "巡检时间") String inspectTime,
                                     @RequestParam @ApiParam(value = "巡检人员id") String inspectPersonId,
                                     @RequestParam @ApiParam(value = "问题描述") String description,
                                     @RequestParam(value = "img1") @ApiParam(value = "图片1") MultipartFile img1,
                                     @RequestParam(value = "img2", required = false) @ApiParam(value = "图片2") MultipartFile img2,
                                     @RequestParam(value = "img3", required = false) @ApiParam(value = "图片3") MultipartFile img3) throws IOException, ParseException {
        dailyInspectionService.edit(id,inspectTime,inspectPersonId,description,img1,img2,img3);
        return CommonResult.ok();
    }

    /**
     * 删除日常巡检
     *
     * @author newspaper
     * @date  2024/01/08 15:58
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除日常巡检")
    @CommonLog("删除日常巡检")
    @PostMapping("/ai/dailyInspection/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DailyInspectionIdParam> dailyInspectionIdParamList) {
        dailyInspectionService.delete(dailyInspectionIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取日常巡检详情
     *
     * @author newspaper
     * @date  2024/01/08 15:58
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取日常巡检详情")
    @GetMapping("/ai/dailyInspection/detail")
    public CommonResult<DailyInspectionImageVo> detail(DailyInspectionIdParam dailyInspectionIdParam) {
        return CommonResult.data(dailyInspectionService.detail(dailyInspectionIdParam));
    }
}
