/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.dirtyCapture.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.ai.dirtyCapture.entity.DirtyCapture;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCaptureAddParam;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCaptureEditParam;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCaptureIdParam;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCapturePageParam;
import vip.xiaonuo.modular.ai.dirtyCapture.service.DirtyCaptureService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 设备镜头脏污控制器
 *
 * @author newspaper
 * @date  2024/01/06 11:30
 */
@Api(tags = "设备镜头脏污控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DirtyCaptureController {

    @Resource
    private DirtyCaptureService dirtyCaptureService;

    /**
     * 获取设备镜头脏污分页
     *
     * @author newspaper
     * @date  2024/01/06 11:30
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取设备镜头脏污分页")
    @GetMapping("/ai/dirtyCapture/page")
    public CommonResult<Page<DirtyCapture>> page(DirtyCapturePageParam dirtyCapturePageParam) {
        return CommonResult.data(dirtyCaptureService.page(dirtyCapturePageParam));
    }

    /**
     * 添加设备镜头脏污
     *
     * @author newspaper
     * @date  2024/01/06 11:30
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加设备镜头脏污")
    @CommonLog("添加设备镜头脏污")
    @PostMapping("/ai/dirtyCapture/add")
    public CommonResult<String> add(@RequestBody @Valid DirtyCaptureAddParam dirtyCaptureAddParam) {
        dirtyCaptureService.add(dirtyCaptureAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑设备镜头脏污
     *
     * @author newspaper
     * @date  2024/01/06 11:30
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑设备镜头脏污")
    @CommonLog("编辑设备镜头脏污")
    @PostMapping("/ai/dirtyCapture/edit")
    public CommonResult<String> edit(@RequestBody @Valid DirtyCaptureEditParam dirtyCaptureEditParam) {
        dirtyCaptureService.edit(dirtyCaptureEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除设备镜头脏污
     *
     * @author newspaper
     * @date  2024/01/06 11:30
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除设备镜头脏污")
    @CommonLog("删除设备镜头脏污")
    @PostMapping("/ai/dirtyCapture/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DirtyCaptureIdParam> dirtyCaptureIdParamList) {
        dirtyCaptureService.delete(dirtyCaptureIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取设备镜头脏污详情
     *
     * @author newspaper
     * @date  2024/01/06 11:30
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取设备镜头脏污详情")
    @GetMapping("/ai/dirtyCapture/detail")
    public CommonResult<DirtyCapture> detail(@Valid DirtyCaptureIdParam dirtyCaptureIdParam) {
        return CommonResult.data(dirtyCaptureService.detail(dirtyCaptureIdParam));
    }
}
