/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.speciesCapture.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEvents;
import vip.xiaonuo.modular.ai.followEvents.service.FollowEventsService;
import vip.xiaonuo.modular.ai.personCapture.entity.PersonCapture;
import vip.xiaonuo.modular.ai.speciesCapture.entity.SpeciesCapture;
import vip.xiaonuo.modular.ai.speciesCapture.mapper.SpeciesCaptureMapper;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCaptureAddParam;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCaptureEditParam;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCaptureIdParam;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCapturePageParam;
import vip.xiaonuo.modular.ai.speciesCapture.service.SpeciesCaptureService;

import java.util.List;

/**
 * 外来生物识别Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/06 11:32
 **/
@Service
public class SpeciesCaptureServiceImpl extends ServiceImpl<SpeciesCaptureMapper, SpeciesCapture> implements SpeciesCaptureService {

    @Autowired
    private FollowEventsService followEventsService;

    @Override
    public Page<SpeciesCapture> page(SpeciesCapturePageParam speciesCapturePageParam) {
        QueryWrapper<SpeciesCapture> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(speciesCapturePageParam.getStartCapTime()) && ObjectUtil.isNotEmpty(speciesCapturePageParam.getEndCapTime())) {
            queryWrapper.lambda().between(SpeciesCapture::getCapTime, speciesCapturePageParam.getStartCapTime(), speciesCapturePageParam.getEndCapTime());
        }
        if(ObjectUtil.isAllNotEmpty(speciesCapturePageParam.getSortField(), speciesCapturePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(speciesCapturePageParam.getSortOrder());
            queryWrapper.orderBy(true, speciesCapturePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(speciesCapturePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByDesc(SpeciesCapture::getCapTime);
        }
        queryWrapper.lambda().eq(SpeciesCapture::getOrgId,speciesCapturePageParam.getOrgId());
        Page<SpeciesCapture> page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        List<SpeciesCapture> records = page.getRecords();
        for (SpeciesCapture record : records) {
            long count = followEventsService.count(new QueryWrapper<FollowEvents>().lambda()
                    .eq(FollowEvents::getEventId, record.getId())
                    .eq(FollowEvents::getEventType, 3).eq(FollowEvents::getUserId, StpLoginUserUtil.getLoginUser().getId()));
            if (count > 0){
                record.setIsFollow(1);
            }else {
                record.setIsFollow(0);
            }
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(SpeciesCaptureAddParam speciesCaptureAddParam) {
        SpeciesCapture speciesCapture = BeanUtil.toBean(speciesCaptureAddParam, SpeciesCapture.class);
        this.save(speciesCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(SpeciesCaptureEditParam speciesCaptureEditParam) {
        SpeciesCapture speciesCapture = this.queryEntity(speciesCaptureEditParam.getId());
        BeanUtil.copyProperties(speciesCaptureEditParam, speciesCapture);
        this.updateById(speciesCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<SpeciesCaptureIdParam> speciesCaptureIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(speciesCaptureIdParamList, SpeciesCaptureIdParam::getId));
    }

    @Override
    public SpeciesCapture detail(SpeciesCaptureIdParam speciesCaptureIdParam) {
        return this.queryEntity(speciesCaptureIdParam.getId());
    }

    @Override
    public SpeciesCapture queryEntity(String id) {
        SpeciesCapture speciesCapture = this.getById(id);
        if(ObjectUtil.isEmpty(speciesCapture)) {
            throw new CommonException("外来生物识别不存在，id值为：{}", id);
        }
        return speciesCapture;
    }
}
