/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.peoplelist.service.impl;

import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.common.util.UploadImage;
import vip.xiaonuo.dev.modular.file.service.DevFileService;
import vip.xiaonuo.modular.base.department.entity.Department;
import vip.xiaonuo.modular.base.department.mapper.DepartmentMapper;
import vip.xiaonuo.modular.base.peoplelist.entity.PeopleList;
import vip.xiaonuo.modular.base.peoplelist.mapper.PeopleListMapper;
import vip.xiaonuo.modular.base.peoplelist.param.PeopleListIdParam;
import vip.xiaonuo.modular.base.peoplelist.param.PeopleListPageParam;
import vip.xiaonuo.modular.base.peoplelist.service.PeopleListService;
import vip.xiaonuo.sys.modular.user.entity.SysUser;
import vip.xiaonuo.sys.modular.user.mapper.SysUserMapper;

import java.io.IOException;
import java.util.List;

/**
 * people_listService接口实现类
 *
 * @author wang
 * @date  2023/12/28 16:19
 **/
@Service
public class PeopleListServiceImpl extends ServiceImpl<PeopleListMapper, PeopleList> implements PeopleListService {

    @Autowired
    private DevFileService devFileService;
    @Autowired
    private DepartmentMapper departmentMapper;
    @Autowired
    private UploadImage uploadImage;
    @Autowired
    private SysUserMapper userMapper;

    @Override
    public Page<PeopleList> page(PeopleListPageParam peopleListPageParam) {
        QueryWrapper<PeopleList> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", peopleListPageParam.getOrgId());
        if(ObjectUtil.isNotEmpty(peopleListPageParam.getUserName())) {
            queryWrapper.lambda().like(PeopleList::getUserName, peopleListPageParam.getUserName());
        }
        if(ObjectUtil.isNotEmpty(peopleListPageParam.getNickName())) {
            queryWrapper.lambda().like(PeopleList::getNickName, peopleListPageParam.getNickName());
        }
        if(ObjectUtil.isAllNotEmpty(peopleListPageParam.getSortField(), peopleListPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(peopleListPageParam.getSortOrder());
            queryWrapper.orderBy(true, peopleListPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(peopleListPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PeopleList::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(String userName, String nickName, Integer sex, String idCard, String phone, String publicPhone,
                    String province, String address, String entryDate, Integer workStatus, String job, String department,
                    String workLocation, MultipartFile imgUrl,String orgId) throws IOException {
        if (this.count(new QueryWrapper<PeopleList>().lambda().eq(PeopleList::getNickName,nickName).eq(PeopleList::getOrgId,orgId)) > 0) {
            throw new CommonException("存在重复工号!");
        }
        if (this.count(new QueryWrapper<PeopleList>().lambda().eq(PeopleList::getIdCard,idCard).eq(PeopleList::getOrgId,orgId)) > 0) {
            throw new CommonException("身份证号已存在!");
        }
        PeopleList people = new PeopleList();
        people.setNickName(nickName);
        people.setUserName(userName);
        people.setSex(sex);
        people.setIdCard(idCard);
        people.setPublicPhone(publicPhone);
        people.setProvince(province);
        people.setAddress(address);
        people.setEntryDate(entryDate);
        people.setWorkStatus(workStatus);
        people.setJob(job);
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", department);
        people.setDepartment(department);
        people.setDepartmentName(departmentMapper.selectOne(queryWrapper).getDepartmentName());
        people.setWorkLocation(workLocation);
        people.setPhone(phone);
        people.setOrgId(orgId);

        SysUser sysUser = new SysUser();
        sysUser.setAccount(nickName);

        if (ObjectUtil.isNotEmpty(imgUrl)){
            String uploadReturnUrl = uploadImage.getImageCom(imgUrl);
            people.setImgUrl(uploadReturnUrl);
            sysUser.setAvatar(uploadReturnUrl);
        }
        sysUser.setName(userName);
        sysUser.setPhone(phone);
        userMapper.insert(sysUser);
        this.save(people);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(String id,String userName, String nickName, Integer sex, String idCard, String phone, String publicPhone,
                     String province, String address, String entryDate, Integer workStatus, String job, String department,
                     String workLocation, MultipartFile imgUrl) throws IOException {
        PeopleList people = this.queryEntity(id);
        if (this.count(new QueryWrapper<PeopleList>().lambda().eq(PeopleList::getNickName,nickName).ne(PeopleList::getId,id)) > 0) {
            throw new CommonException("存在重复工号!");
        }
        if (this.count(new QueryWrapper<PeopleList>().lambda().eq(PeopleList::getIdCard,idCard).ne(PeopleList::getId,id)) > 0) {
            throw new CommonException("身份证号已存在!");
        }
        people.setNickName(nickName);
        people.setUserName(userName);
        people.setSex(sex);
        people.setIdCard(idCard);
        people.setPublicPhone(publicPhone);
        people.setProvince(province);
        people.setAddress(address);
        people.setEntryDate(entryDate);
        people.setWorkStatus(workStatus);
        people.setJob(job);
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", department);
        people.setDepartment(department);
        people.setDepartmentName(departmentMapper.selectOne(queryWrapper).getDepartmentName());
        people.setWorkLocation(workLocation);
        people.setPhone(phone);
        if (ObjectUtil.isNotEmpty(imgUrl)){
            String uploadReturnUrl = uploadImage.getImageCom(imgUrl);
            people.setImgUrl(uploadReturnUrl);
        }
        this.updateById(people);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PeopleListIdParam> peopleListIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(peopleListIdParamList, PeopleListIdParam::getId));
    }

    @Override
    public PeopleList detail(PeopleListIdParam peopleListIdParam) {
        return this.queryEntity(peopleListIdParam.getId().toString());
    }

    @Override
    public PeopleList queryEntity(String id) {
        PeopleList peopleList = this.getById(id);
        if(ObjectUtil.isEmpty(peopleList)) {
            throw new CommonException("people_list不存在，id值为：{}", id);
        }
        return peopleList;
    }
}
