/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.pigFarm.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.base.pigFarm.entity.PigFarm;
import vip.xiaonuo.modular.base.pigFarm.param.PigFarmAddParam;
import vip.xiaonuo.modular.base.pigFarm.param.PigFarmEditParam;
import vip.xiaonuo.modular.base.pigFarm.param.PigFarmIdParam;
import vip.xiaonuo.modular.base.pigFarm.service.PigFarmService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;

/**
 * 猪场控制器
 *
 * @author newspaper
 * @date  2024/01/25 15:11
 */
@Api(tags = "猪场控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PigFarmController {

    @Resource
    private PigFarmService pigFarmService;

    /**
     * 获取猪场分页
     *
     * @author newspaper
     * @date  2024/01/25 15:11
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取猪场")
    @GetMapping("/base/pigFarm/getSelf")
    public CommonResult<PigFarm> getSelf(OrgIdParam orgIdParam) {
        return CommonResult.data(pigFarmService.getSelf(orgIdParam));
    }

    /**
     * 添加猪场
     *
     * @author newspaper
     * @date  2024/01/25 15:11
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加猪场")
    @CommonLog("添加猪场")
    @PostMapping("/base/pigFarm/add")
    public CommonResult<String> add(@RequestParam @ApiParam(value = "名称") String name,
                                    @RequestParam @ApiParam(value = "规模") String size,
                                    @RequestParam @ApiParam(value = "所在区域") String location,
                                    @RequestParam @ApiParam(value = "详细地址") String detailLocation,
                                    @RequestParam @ApiParam(value = "组织id") String orgId,
                                    @RequestParam(value = "imgUrl",required = false) @ApiParam(value = "猪场图片") MultipartFile imgUrl) throws IOException {
        pigFarmService.add(name,size,location,detailLocation,orgId,imgUrl);
        return CommonResult.ok();
    }

    /**
     * 编辑猪场
     *
     * @author newspaper
     * @date  2024/01/25 15:11
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑猪场")
    @CommonLog("编辑猪场")
    @PostMapping("/base/pigFarm/edit")
    public CommonResult<String> edit(@RequestParam @ApiParam(value = "id") String id,
                                     @RequestParam @ApiParam(value = "规模") String size,
                                     @RequestParam @ApiParam(value = "所在区域") String location,
                                     @RequestParam @ApiParam(value = "详细地址") String detailLocation,
                                     @RequestParam @ApiParam(value = "组织id") String orgId,
                                     @RequestParam(value = "imgUrl",required = false) @ApiParam(value = "猪场图片") MultipartFile imgUrl) throws IOException {
        pigFarmService.edit(id,size,location,detailLocation,orgId,imgUrl);
        return CommonResult.ok();
    }

    /**
     * 删除猪场
     *
     * @author newspaper
     * @date  2024/01/25 15:11
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除猪场")
    @CommonLog("删除猪场")
    @PostMapping("/base/pigFarm/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<PigFarmIdParam> pigFarmIdParamList) {
        pigFarmService.delete(pigFarmIdParamList);
        return CommonResult.ok();
    }
}
