/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigbatch.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;


import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.bi.pigbatch.entity.PigBatch;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchAddParam;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchEditParam;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchIdParam;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchPageParam;
import vip.xiaonuo.modular.bi.pigbatch.service.PigBatchService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 猪只批次控制器
 *
 * @author wang
 * @date  2023/12/06 14:53
 */
@Api(tags = "猪只批次控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PigBatchController {

    @Resource
    private PigBatchService pigBatchService;

    /**
     * 获取猪只批次分页
     *
     * @author wang
     * @date  2023/12/06 14:53
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取猪只批次分页")
    @GetMapping("/bi/pigbatch/page")
    public CommonResult<Page<PigBatch>> page(PigBatchPageParam pigBatchPageParam) {
        return CommonResult.data(pigBatchService.page(pigBatchPageParam));
    }

    /**
     * 添加猪只批次
     *
     * @author wang
     * @date  2023/12/06 14:53
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加猪只批次")
    @CommonLog("添加猪只批次")
    @PostMapping("/bi/pigbatch/add")
    public CommonResult<String> add(@RequestBody @Valid PigBatchAddParam pigBatchAddParam) {
        return pigBatchService.add(pigBatchAddParam);
    }

    /**
     * 编辑猪只批次
     *
     * @author wang
     * @date  2023/12/06 14:53
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑猪只批次")
    @CommonLog("编辑猪只批次")
    @PostMapping("/bi/pigbatch/edit")
    public CommonResult<String> edit(@RequestBody @Valid PigBatchEditParam pigBatchEditParam) {
        return pigBatchService.edit(pigBatchEditParam);
    }

    /**
     * 删除猪只批次
     *
     * @author wang
     * @date  2023/12/06 14:53
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除猪只批次")
    @CommonLog("删除猪只批次")
    @PostMapping("/bi/pigbatch/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<PigBatchIdParam> pigBatchIdParamList) {
        pigBatchService.delete(pigBatchIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取猪只批次详情
     *
     * @author wang
     * @date  2023/12/06 14:53
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取猪只批次详情")
    @GetMapping("/bi/pigbatch/detail")
    public CommonResult<PigBatch> detail(@Valid PigBatchIdParam pigBatchIdParam) {
        return CommonResult.data(pigBatchService.detail(pigBatchIdParam));
    }
}
