/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigcure.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.bi.pigcure.entity.PigCure;
import vip.xiaonuo.modular.bi.pigcure.param.PigCureAddParam;
import vip.xiaonuo.modular.bi.pigcure.param.PigCureEditParam;
import vip.xiaonuo.modular.bi.pigcure.param.PigCureIdParam;
import vip.xiaonuo.modular.bi.pigcure.param.PigCurePageParam;
import vip.xiaonuo.modular.bi.pigcure.service.PigCureService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 防治信息控制器
 *
 * @author wang
 * @date  2023/12/08 10:03
 */
@Api(tags = "防治信息控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PigCureController {

    @Resource
    private PigCureService pigCureService;

    /**
     * 获取防治信息分页
     *
     * @author wang
     * @date  2023/12/08 10:03
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取防治信息分页")
    @GetMapping("/bi/pigcure/page")
    public CommonResult<Page<PigCure>> page(PigCurePageParam pigCurePageParam) {
        return CommonResult.data(pigCureService.page(pigCurePageParam));
    }

    /**
     * 添加防治信息
     *
     * @author wang
     * @date  2023/12/08 10:03
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加防治信息")
    @CommonLog("添加防治信息")
    @PostMapping("/bi/pigcure/add")
    public CommonResult<String> add(@RequestBody @Valid PigCureAddParam pigCureAddParam) {
        pigCureService.add(pigCureAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑防治信息
     *
     * @author wang
     * @date  2023/12/08 10:03
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑防治信息")
    @CommonLog("编辑防治信息")
    @PostMapping("/bi/pigcure/edit")
    public CommonResult<String> edit(@RequestBody @Valid PigCureEditParam pigCureEditParam) {
        pigCureService.edit(pigCureEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除防治信息
     *
     * @author wang
     * @date  2023/12/08 10:03
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除防治信息")
    @CommonLog("删除防治信息")
    @PostMapping("/bi/pigcure/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<PigCureIdParam> pigCureIdParamList) {
        pigCureService.delete(pigCureIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取防治信息详情
     *
     * @author wang
     * @date  2023/12/08 10:03
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取防治信息详情")
    @GetMapping("/bi/pigcure/detail")
    public CommonResult<PigCure> detail(@Valid PigCureIdParam pigCureIdParam) {
        return CommonResult.data(pigCureService.detail(pigCureIdParam));
    }
}
