/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigcure.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.bi.pigcure.entity.PigCure;
import vip.xiaonuo.modular.bi.pigcure.mapper.PigCureMapper;
import vip.xiaonuo.modular.bi.pigcure.param.PigCureAddParam;
import vip.xiaonuo.modular.bi.pigcure.param.PigCureEditParam;
import vip.xiaonuo.modular.bi.pigcure.param.PigCureIdParam;
import vip.xiaonuo.modular.bi.pigcure.param.PigCurePageParam;
import vip.xiaonuo.modular.bi.pigcure.service.PigCureService;

import java.util.Date;
import java.util.List;

/**
 * 防治信息Service接口实现类
 *
 * @author wang
 * @date  2023/12/08 10:03
 **/
@Service
public class PigCureServiceImpl extends ServiceImpl<PigCureMapper, PigCure> implements PigCureService {

    @Override
    public Page<PigCure> page(PigCurePageParam pigCurePageParam) {
        QueryWrapper<PigCure> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigCurePageParam.getEartag())) {
            queryWrapper.lambda().like(PigCure::getEartag, pigCurePageParam.getEartag());
        }
        if(ObjectUtil.isNotEmpty(pigCurePageParam.getStartCureDate()) && ObjectUtil.isNotEmpty(pigCurePageParam.getEndCureDate())) {
            queryWrapper.lambda().between(PigCure::getCureDate, pigCurePageParam.getStartCureDate(), pigCurePageParam.getEndCureDate());
        }
        if(ObjectUtil.isAllNotEmpty(pigCurePageParam.getSortField(), pigCurePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigCurePageParam.getSortOrder());
            queryWrapper.orderBy(true, pigCurePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigCurePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigCure::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PigCureAddParam pigCureAddParam) {
        PigCure pigCure = BeanUtil.toBean(pigCureAddParam, PigCure.class);
        pigCure.setCreateDate(new Date());
        this.save(pigCure);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PigCureEditParam pigCureEditParam) {
        PigCure pigCure = this.queryEntity(pigCureEditParam.getId());
        BeanUtil.copyProperties(pigCureEditParam, pigCure);
        this.updateById(pigCure);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigCureIdParam> pigCureIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigCureIdParamList, PigCureIdParam::getId));
    }

    @Override
    public PigCure detail(PigCureIdParam pigCureIdParam) {
        return this.queryEntity(pigCureIdParam.getId());
    }

    @Override
    public PigCure queryEntity(Integer id) {
        PigCure pigCure = this.getById(id);
        if(ObjectUtil.isEmpty(pigCure)) {
            throw new CommonException("防治信息不存在，id值为：{}", id);
        }
        return pigCure;
    }
}
