/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.env.envThreshold.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;

import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.env.envThreshold.entity.EnvThreshold;
import vip.xiaonuo.modular.env.envThreshold.entity.vo.EnvThresholdListVo;
import vip.xiaonuo.modular.env.envThreshold.param.*;
import vip.xiaonuo.modular.env.envThreshold.service.EnvThresholdService;
import vip.xiaonuo.modular.warning.warningManager.entity.vo.WarningManagerVo;
import vip.xiaonuo.modular.warning.warningManager.param.WarningManagerTypeParam;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * 环控阈值控制器
 *
 * @author newspaper
 * @date  2023/12/25 15:38
 */
@Api(tags = "环控阈值控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class EnvThresholdController {

    @Resource
    private EnvThresholdService envThresholdService;

    /**
     * 根据单元ID获取阈值
     *
     * @author newspaper
     * @date  2023/12/25 15:45
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取环控阈值")
    @GetMapping("/env/envThreshold/getThreshold")
    public CommonResult<List<EnvThresholdListVo>> getThreshold(EnvThresholdOrgIdParam envThresholdOrgIdParam) {
        return CommonResult.data(envThresholdService.getThreshold(envThresholdOrgIdParam));
    }

    /**
     * 编辑环控阈值
     *
     * @author newspaper
     * @date  2023/12/25 15:45
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("编辑环控阈值")
    @PostMapping("/env/envThreshold/saveThreshold")
    public CommonResult<String> saveThreshold(@RequestBody @Valid EnvThresholdSaveParam envThresholdSaveParam) {
        envThresholdService.saveThreshold(envThresholdSaveParam);
        return CommonResult.ok();
    }


    /**
     * 修改环控阈值状态
     *
     * @author newspaper
     * @date  2023/12/25 15:45
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("修改环控阈值状态")
    @PostMapping("/env/envThreshold/modifyThresholdStatus")
    public CommonResult<String> modifyThresholdStatus(@RequestBody @Valid EnvThresholdModifyParam envThresholdModifyParam) {
        envThresholdService.modifyThresholdStatus(envThresholdModifyParam);
        return CommonResult.ok();
    }

    /**
     * 同步环控阈值
     *
     * @author newspaper
     * @date  2023/12/25 15:45
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("同步环控阈值")
    @PostMapping("/env/envThreshold/syncThreshold")
    public CommonResult<String> syncThreshold(@RequestBody @Valid EnvThresholdSyncParam envThresholdSyncParam) {
        envThresholdService.syncThreshold(envThresholdSyncParam);
        return CommonResult.ok();
    }



}
